/*
 * Copyright (c) 2011, Noel Lemouel
 * Based on the work from Teunis van Beelen, see credits below
 * GPL v2.0 applies
 */

/**************************************************************************
*
* Author: Teunis van Beelen
*
* Copyright (C) 2005, 2006, 2007, 2008, 2009 Teunis van Beelen
*
* teuniz@gmail.com
*
***************************************************************************
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation version 2 of the License.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*
***************************************************************************
*
* This version of GPL is at http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
*
***************************************************************************
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

#include <windows.h>

#ifdef __cplusplus
} /* extern "C" */
#endif

static HANDLE dev_h[256];

static char dev_cstrg[64];
static char dev_name[32];

int serial_open(unsigned int dev_num, int speed)
{
   unsigned char s[4];

   DCB dev_cblk;
   COMMTIMEOUTS dev_t_out;

   if(dev_num > 255)
   {
      printf("invalid COM port# [range => 0-255]\n");
      return -1;
   }

   strcpy(dev_name, "\\\\.\\COM");
   strcat(dev_name, itoa(dev_num, s, 10));

//   printf("%s\n", dev_name);

   switch(speed)
   {
      case     110 : strcpy(dev_cstrg, "baud=110");
                   break;
      case     300 : strcpy(dev_cstrg, "baud=300");
                   break;
      case     600 : strcpy(dev_cstrg, "baud=600");
                   break;
      case    1200 : strcpy(dev_cstrg, "baud=1200");
                   break;
      case    2400 : strcpy(dev_cstrg, "baud=2400");
                   break;
      case    4800 : strcpy(dev_cstrg, "baud=4800");
                   break;
      case    9600 : strcpy(dev_cstrg, "baud=9600");
                   break;
      case   19200 : strcpy(dev_cstrg, "baud=19200");
                   break;
      case   38400 : strcpy(dev_cstrg, "baud=38400");
                   break;
      case   57600 : strcpy(dev_cstrg, "baud=57600");
                   break;
      case  115200 : strcpy(dev_cstrg, "baud=115200");
                   break;
      case  128000 : strcpy(dev_cstrg, "baud=128000");
                   break;
      case  256000 : strcpy(dev_cstrg, "baud=256000");
                   break;
      default      : printf("invalid baudrate [%d]\n", speed);
                   return -1;
                   break;
   }

   strcat(dev_cstrg, " data=8 parity=N stop=1");

   dev_h[dev_num] = CreateFileA(dev_name,
                        GENERIC_READ|GENERIC_WRITE,
                        0,                          /* no share  */
                        NULL,                       /* no security */
                        OPEN_EXISTING,
                        0,                          /* no threads */
                        NULL);                      /* no templates */

   if(dev_h[dev_num]==INVALID_HANDLE_VALUE)
   {
      printf("failed to open COM%d\n", dev_num);
      return -1;
   }

   /* DCB */

   memset(&dev_cblk, 0x00, sizeof(dev_cblk));        /* clear the new struct  */
   dev_cblk.DCBlength = sizeof(dev_cblk);

   if(!BuildCommDCBA(dev_cstrg, &dev_cblk))
   {
      printf("failed to set COM%d DCB settings [BuildCommDCBA]\n", dev_num);
      CloseHandle(dev_h[dev_num]);
      return -1;
   }

   if(!SetCommState(dev_h[dev_num], &dev_cblk))
   {
      printf("failed to configure COM%d [SetCommState]\n", dev_num);
      CloseHandle(dev_h[dev_num]);
      return -1;
   }

   /* COMMTIMEOUTS */

   dev_t_out.ReadIntervalTimeout         = MAXDWORD;
   dev_t_out.ReadTotalTimeoutMultiplier  = 0;
   dev_t_out.ReadTotalTimeoutConstant    = 0;
   dev_t_out.WriteTotalTimeoutMultiplier = 0;
   dev_t_out.WriteTotalTimeoutConstant   = 0;

   if(!SetCommTimeouts(dev_h[dev_num], &dev_t_out))
   {
      printf("failed to apply COM%d time-out settings\n", dev_num);
      CloseHandle(dev_h[dev_num]);
      return -1;
   }

   return 0;
}

int serial_tx(int dev_num, unsigned char* data, int size)
{
  int n;

  if(WriteFile(dev_h[dev_num], data, size, (LPDWORD)((void *)&n), NULL))
  {
    return n;
  }

  return -1;
}

int serial_rx(int dev_num, unsigned char* data, int size)
{
  int n;

  /* if(size>4096)  size = 4096; */

/* added the void pointer cast, otherwise gcc will complain about */
/* "warning: dereferencing type-punned pointer will break strict aliasing rules" */

  ReadFile(dev_h[dev_num], data, size, (LPDWORD)((void *)&n), NULL);

  return n;
}

void serial_close(int dev_num)
{
  CloseHandle(dev_h[dev_num]);
}

/*
int serial_tx_8(int dev_num, unsigned char c)
{
  int n;

  WriteFile(dev_h[dev_num], &c, 1, (LPDWORD)((void *)&n), NULL);

  if(n<0)  return -1;

  return 0;
}
*/

/*
int IsCTSEnabled(int dev_num)
{
  int status;

  GetCommModemStatus(dev_h[dev_num], (LPDWORD)((void *)&status));

  if(status&MS_CTS_ON) return(1);
  else return(0);
}
*/

#ifdef __cplusplus
} /* extern "C" */
#endif

